import argparse
import logging
import os
import subprocess
import sys

import ruamel.yaml

from . import Collection, Entry, TemplatingError


def edit_file_in_external_editor(filepath):
    # First we try to use $EDITOR
    editor = os.environ.get('EDITOR')
    if editor:
        try:
            subprocess.call([editor, filepath])
        except FileNotFoundError as e:
            if editor in str(e):
                logging.info(
                    'Could not open file {} with $EDITOR (currently {})'
                    .format(
                        filepath, editor
                    )
                )
            else:
                logging.warning("Could not open file {}".format(filepath))
                return False
        else:
            return True
    # then we try to use sensible-editor (which should be available on
    # debian and derivatives)
    try:
        subprocess.call(['sensible-editor', filepath])
    except FileNotFoundError as e:
        if 'sensible-editor' in e.strerror:
            logging.debug(
                "Could not open file {} with editor: sensible-editor".format(
                    filepath
                )
            )
        else:
            logging.warning("Could not open file {}".format(filepath))
            return False
    else:
        return True
    # and finally we fallback to vi, because ed is the standard editor,
    # but that would be way too cruel, and vi is also in posix
    try:
        subprocess.call(['vi', filepath])
    except FileNotFoundError as e:
        if 'vi' in e.strerror:
            logging.warning(
                "Could not open file {} with any known editor".format(filepath)
            )
            return False
        else:
            logging.warning("Could not open file {}".format(filepath))
            return False
    else:
        return True


def _get_first_docstring_line(obj):
    try:
        return obj.__doc__.split('\n')[1].strip()
    except (AttributeError, IndexError):
        return None


class MainCommand:
    commands = ()

    def _main(self, args):
        self.parser.print_help()

    def main(self):
        desc = _get_first_docstring_line(self)
        self.parser = argparse.ArgumentParser(description=desc)
        self.parser.set_defaults(func=self._main)
        self.subparsers = self.parser.add_subparsers()
        for name, sub in self.commands:
            sub_help = _get_first_docstring_line(sub)
            s_parser = self.subparsers.add_parser(name, help=sub_help)
            for arg in sub.arguments:
                s_parser.add_argument(*arg[0], **arg[1])
            s_parser.set_defaults(func=sub._main)
        self.args = self.parser.parse_args()
        self.args.func(self.args)


class Command:
    def __init__(self, collection_class=Collection, entry_class=Entry):
        self.collection_class = collection_class
        self.entry_class = entry_class

    def _main(self, args):
        self.args = args
        self.main()


class New(Command):
    """
    Create a new entry
    """
    arguments = [
        (
            ['--collection', '-c'],
            dict(help='The collection to work on (default .)'),
        ),
        (
            ['--no-git'],
            dict(
                help="Don't add the new entry to git",
                action="store_false",
                dest='git',
            ),
        ),
    ]

    def main(self):
        collection = self.collection_class(self.args.collection)
        new_entry = self.entry_class(collection)
        collection.save_entries([new_entry])
        filepath = os.path.join(collection.itemdir, new_entry.fname)
        if edit_file_in_external_editor(filepath):
            collection.update_cache([filepath])
            if self.args.git:
                collection.git_add_files([filepath])
        saved_entry = collection.entry_from_eid(new_entry.eid)
        print(saved_entry)


class Edit(Command):
    """
    Edit a lesana entry
    """
    arguments = [
        (
            ['--collection', '-c'],
            dict(help='The collection to work on (default .)'),
        ),
        (
            ['--no-git'],
            dict(
                help="Don't add the new entry to git",
                action="store_false",
                dest='git',
            ),
        ),
        (['eid'], dict(help='eid of an entry to edit',)),
    ]

    def main(self):
        collection = self.collection_class(self.args.collection)
        entries = collection.entries_from_short_eid(self.args.eid)
        if len(entries) > 1:
            return "{} is not an unique eid".format(self.args.eid)
        if not entries:
            return "Could not find an entry with eid starting with: {}".format(
                self.args.eid
            )
        entry = entries[0]
        # update the entry before editing it
        entry.auto()
        collection.save_entries([entry])
        # and then edit the updated file
        filepath = os.path.join(collection.itemdir, entry.fname)
        if edit_file_in_external_editor(filepath):
            collection.update_cache([filepath])
            if self.args.git:
                collection.git_add_files([filepath])
        saved_entry = collection.entry_from_eid(entry.eid)
        print(saved_entry)


class Show(Command):
    """
    Show a lesana entry
    """
    arguments = [
        (
            ['--collection', '-c'],
            dict(help='The collection to work on (default .)'),
        ),
        (
            ['--template', '-t'],
            dict(help='Use the specified template to display results.',),
        ),
        (['eid'], dict(help='eid of an entry to edit',)),
    ]

    def main(self):
        collection = self.collection_class(self.args.collection)
        entries = collection.entries_from_short_eid(self.args.eid)
        if len(entries) > 1:
            return "{} is not an unique eid".format(self.args.eid)
        if not entries:
            return "Could not find an entry with eid starting with: {}".format(
                self.args.eid
            )
        entry = entries[0]
        if self.args.template:
            try:
                print(entry.render(self.args.template))
            except TemplatingError as e:
                logging.error("{}".format(e))
                sys.exit(1)
        else:
            print(entry.yaml_data)


class Index(Command):
    """
    Index entries in a lesana collection
    """
    arguments = [
        (
            ['--collection', '-c'],
            dict(help='The collection to work on (default .)'),
        ),
        (
            ['--reset'],
            dict(
                action='store_true',
                help='Delete the existing index and reindex from scratch.',
            ),
        ),
        (
            ['files'],
            dict(
                help='List of files to index (default: everything)',
                default=None,
                nargs='*',
            ),
        ),
    ]

    def main(self):
        collection = self.collection_class(self.args.collection)
        if self.args.files:
            files = (os.path.basename(f) for f in self.args.files)
        else:
            files = None
        indexed = collection.update_cache(
            fnames=files,
            reset=self.args.reset
        )
        print("Found and indexed {} entries".format(indexed))


class Search(Command):
    """
    Search for entries
    """
    arguments = [
        (
            ['--collection', '-c'],
            dict(help='The collection to work on (default .)'),
        ),
        (
            ['--template', '-t'],
            dict(help='Template to use when displaying results',),
        ),
        (['--offset'], dict(type=int,)),
        (['--pagesize'], dict(type=int,)),
        (
            ['--all'],
            dict(action='store_true', help='Return all available results'),
        ),
        (
            ['--sort'],
            dict(action='append', help='Sort results by a sortable field'),
        ),
        (
            ['query'],
            dict(help='Xapian query to search in the collection', nargs='+'),
        ),
    ]

    def main(self):
        # TODO: implement "searching" for everything
        if self.args.offset:
            logging.warning(
                "offset exposes an internal knob and MAY BE REMOVED "
                + "from a future release of lesana"  # noqa: W503
            )
        if self.args.pagesize:
            logging.warning(
                "pagesize exposes an internal knob and MAY BE REMOVED "
                + "from a future release of lesana"  # noqa: W503
            )
        offset = self.args.offset or 0
        pagesize = self.args.pagesize or 12
        collection = self.collection_class(self.args.collection)
        # sorted results require a less efficient full search rather
        # than being able to use the list of all documents.
        if self.args.query == ['*'] and not (
            self.args.sort
            or getattr(collection.settings, 'default_sort', False)
        ):
            results = collection.get_all_documents()
        else:
            collection.start_search(
                ' '.join(self.args.query),
                sort_by=self.args.sort
            )
            if self.args.all:
                results = collection.get_all_search_results()
            else:
                results = collection.get_search_results(offset, pagesize)
        if self.args.template:
            try:
                template = collection.get_template(self.args.template)
                print(template.render(entries=results))
            except TemplatingError as e:
                logging.error("{}".format(e))
                sys.exit(1)
        else:
            for entry in results:
                print("{entry}".format(entry=entry,))


class Export(Command):
    """
    Export entries to a different collection
    """
    arguments = [
        (
            ['--collection', '-c'],
            dict(help='The collection to work on (default .)'),
        ),
        (
            ['--query', '-q'],
            dict(help='Xapian query to search in the collection',),
        ),
        (['destination'], dict(help='The collection to export entries to')),
        (['template'], dict(help='Template to convert entries',)),
    ]

    def main(self):
        collection = self.collection_class(self.args.collection)
        destination = self.collection_class(self.args.destination)
        if not self.args.query:
            results = collection.get_all_documents()
        else:
            collection.start_search(' '.join(self.args.query))
            results = collection.get_all_search_results()
        for entry in results:
            try:
                template = collection.get_template(self.args.template)
                rendered = template.render(entry=entry, **entry.data)
            except TemplatingError as e:
                logging.error("Error converting entry: {}".format(entry))
                logging.error("{}".format(e))
                sys.exit(1)
            try:
                data = ruamel.yaml.load(rendered, ruamel.yaml.RoundTripLoader)
            except ruamel.yaml.YAMLError as e:
                logging.error("Error converting entry: {}".format(entry))
                logging.error("{}".format(e))
                sys.exit(1)
            e = self.entry_class(destination, data=data)
            destination.save_entries([e])


class Init(Command):
    """
    Initialize a lesana collection
    """
    arguments = [
        (
            ['--collection', '-c'],
            dict(help='The directory to work on (default .)', default='.'),
        ),
        (
            ['--no-git'],
            dict(
                help='Skip setting up git in this directory',
                action="store_false",
                dest='git',
            ),
        ),
    ]

    def main(self):
        self.collection_class.init(
            self.args.collection,
            git_enabled=self.args.git,
            edit_file=edit_file_in_external_editor,
        )


class Remove(Command):
    """
    Remove an entry from a collection
    """
    arguments = [
        (
            ['--collection', '-c'],
            dict(help='The collection to work on (default .)',),
        ),
        (['entries'], dict(help='List of entries to remove', nargs='+',)),
    ]

    def main(self):
        collection = self.collection_class(self.args.collection)
        collection.remove_entries(eids=self.args.entries)


class Update(Command):
    """
    Update a field in multiple entries
    """
    arguments = [
        (
            ['--collection', '-c'],
            dict(help='The collection to work on (default .)',),
        ),
        (['--field', '-f'], dict(help='The field to change',)),
        (['--value', '-t'], dict(help='The value to set',)),
        (
            ['query'],
            dict(help='Xapian query to search in the collection', nargs='+'),
        ),
    ]

    def main(self):
        collection = self.collection_class(self.args.collection)
        collection.update_field(
            ' '.join(self.args.query),
            field=self.args.field,
            value=self.args.value,
        )



import datetime
import decimal
import unittest

import xapian

from lesana import types


class testTypes(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def _get_field_def(self, type_name):
        return {
            'type': type_name,
            'name': 'test_field',
        }

    def test_base(self):
        checker = types.LesanaType(self._get_field_def('base'), {})

        # The base class does not implement empty nor load
        with self.assertRaises(NotImplementedError):
            checker.empty()

        with self.assertRaises(NotImplementedError):
            checker.load("")

    def test_string(self):
        checker = types.LesanaString(self._get_field_def('string'), {})

        s = checker.empty()
        self.assertEqual(s, "")

        s = checker.load("Hello World!")
        self.assertEqual(s, "Hello World!")

        s = checker.load(None)
        self.assertEqual(s, None)

        v = checker.auto("Hello World!")
        self.assertEqual(v, "Hello World!")

    def test_text(self):
        checker = types.LesanaText(self._get_field_def('text'), {})

        s = checker.empty()
        self.assertEqual(s, "")

        s = checker.load("Hello World!")
        self.assertEqual(s, "Hello World!")

        s = checker.load(None)
        self.assertEqual(s, None)

        v = checker.auto("Hello World!")
        self.assertEqual(v, "Hello World!")

    def test_int(self):
        checker = types.LesanaInt(self._get_field_def('integer'), {})

        v = checker.empty()
        self.assertEqual(v, 0)

        v = checker.load("10")
        self.assertEqual(v, 10)

        v = checker.load(10.5)
        self.assertEqual(v, 10)

        for d in ("ten", "10.5"):
            with self.assertRaises(types.LesanaValueError):
                checker.load(d)

        v = checker.load(None)
        self.assertEqual(v, None)

        v = checker.auto(10)
        self.assertEqual(v, 10)

    def test_datetime_auto_increment(self):
        field_def = self._get_field_def('integer')
        field_def['auto'] = 'increment'
        checker = types.LesanaInt(field_def, {})

        v = checker.empty()
        self.assertEqual(v, 0)

        v = checker.auto(0)
        self.assertEqual(v, 1)

        field_def['increment'] = -1
        checker = types.LesanaInt(field_def, {})
        v = checker.auto(0)
        self.assertEqual(v, -1)

        field_def['increment'] = 0.5
        checker = types.LesanaInt(field_def, {})
        with self.assertLogs() as cm:
            v = checker.auto(0)
        self.assertIn('WARNING', cm.output[0])
        self.assertIn('Invalid configuration value', cm.output[0])
        self.assertEqual(v, 0)

        field_def['auto'] = 'false'
        checker = types.LesanaInt(field_def, {})
        v = checker.auto(0)
        self.assertEqual(v, 0)

    def test_float(self):
        checker = types.LesanaFloat(self._get_field_def('float'), {})

        v = checker.empty()
        self.assertEqual(v, 0.0)

        v = checker.load("10")
        self.assertEqual(v, 10)

        v = checker.load(10.5)
        self.assertEqual(v, 10.5)

        v = checker.load("10.5")
        self.assertEqual(v, 10.5)

        for d in ("ten"):
            with self.assertRaises(types.LesanaValueError):
                checker.load(d)

        v = checker.load(None)
        self.assertEqual(v, None)

        v = checker.auto(10.5)
        self.assertEqual(v, 10.5)

    def test_decimal(self):
        checker = types.LesanaDecimal(self._get_field_def('decimal'), {})

        v = checker.empty()
        self.assertEqual(v, decimal.Decimal(0))

        v = checker.load("10")
        self.assertEqual(v, decimal.Decimal(10))

        v = checker.load(10.5)
        self.assertEqual(v, decimal.Decimal(10.5))

        v = checker.load("10.5")
        self.assertEqual(v, decimal.Decimal(10.5))

        for d in ("ten"):
            with self.assertRaises(types.LesanaValueError):
                checker.load(d)

        v = checker.load(None)
        self.assertEqual(v, None)

        v = checker.auto(decimal.Decimal("10.5"))
        self.assertEqual(v, decimal.Decimal("10.5"))

    def test_timestamp(self):
        checker = types.LesanaTimestamp(self._get_field_def('timestamp'), {})

        v = checker.empty()
        self.assertEqual(v, None)

        now = datetime.datetime.now()
        v = checker.load(now)
        self.assertEqual(v, now)

        v = checker.load("1600000000")
        wanted = datetime.datetime(
            2020, 9, 13, 12, 26, 40, 0,
            datetime.timezone.utc,
        )
        self.assertEqual(v, wanted)

        today = datetime.date.today()
        for d in (
            today,
            "today",
            "2020-13-01", "2020-01-01",
            "2020-01-01 10:00"
        ):
            with self.assertRaises(types.LesanaValueError):
                checker.load(d)

        v = checker.load(None)
        self.assertEqual(v, None)

        v = checker.auto(today)
        self.assertEqual(v, today)

    def test_datetime(self):
        checker = types.LesanaDatetime(self._get_field_def('datetime'), {})

        v = checker.empty()
        self.assertEqual(v, None)

        now = datetime.datetime.now()
        v = checker.load(now)
        self.assertEqual(v, now)

        today = datetime.date.today()
        v = checker.load(today)
        self.assertIsInstance(v, datetime.datetime)
        for part in ('year', 'month', 'day'):
            self.assertEqual(getattr(v, part), getattr(today, part))

        v = checker.load("2020-01-01")
        self.assertEqual(v, datetime.datetime(2020, 1, 1))

        v = checker.load("2020-01-01 10:00")
        self.assertEqual(v, datetime.datetime(2020, 1, 1, 10, 0))

        for d in ("today", "2020-13-01"):
            with self.assertRaises(types.LesanaValueError):
                checker.load(d)

        v = checker.load(None)
        self.assertEqual(v, None)

        v = checker.auto(now)
        self.assertEqual(v, now)

    def test_datetime_auto(self):
        field_def = self._get_field_def('datetime')
        field_def['auto'] = 'creation'
        checker = types.LesanaDatetime(field_def, {})

        now = datetime.datetime.now()
        v = checker.empty()
        self.assertIsInstance(v, datetime.datetime)
        self.assertEqual(v.tzinfo, datetime.timezone.utc)
        self.assertEqual(v.year, now.year)

        field_def['auto'] = False
        checker = types.LesanaDatetime(field_def, {})

        v = checker.empty()
        self.assertEqual(v, None)

        # auto=update fields should also be filled at creation time
        field_def['auto'] = 'update'
        checker = types.LesanaDatetime(field_def, {})
        v = checker.empty()
        self.assertIsInstance(v, datetime.datetime)
        self.assertEqual(v.tzinfo, datetime.timezone.utc)
        self.assertEqual(v.year, now.year)

    def test_datetime_auto_update(self):
        field_def = self._get_field_def('datetime')
        field_def['auto'] = 'update'
        checker = types.LesanaDatetime(field_def, {})

        now = datetime.datetime.now()
        past = datetime.datetime(2016, 12, 10, 21, 2)
        # we pass a date in the past
        v = checker.auto(past)
        self.assertIsInstance(v, datetime.datetime)
        self.assertEqual(v.tzinfo, datetime.timezone.utc)
        # and we want to get a date in the present
        self.assertEqual(v.year, now.year)

        # with auto=False we want our old date instead
        field_def['auto'] = False
        checker = types.LesanaDatetime(field_def, {})

        v = checker.auto(past)
        self.assertEqual(v, past)

        # and the same should happen with auto=creation
        field_def['auto'] = 'creation'
        checker = types.LesanaDatetime(field_def, {})

        v = checker.auto(past)
        self.assertEqual(v, past)

    def test_date(self):
        checker = types.LesanaDate(self._get_field_def('date'), {})

        v = checker.empty()
        self.assertEqual(v, None)

        now = datetime.datetime.now()
        v = checker.load(now)
        self.assertIsInstance(v, datetime.date)
        for part in ('year', 'month', 'day'):
            self.assertEqual(getattr(v, part), getattr(now, part))

        today = datetime.date.today()
        v = checker.load(today)
        self.assertEqual(v, today)

        v = checker.load("2020-01-01")
        self.assertEqual(v, datetime.datetime(2020, 1, 1))

        v = checker.load("2020-01-01 10:00")
        self.assertEqual(v, datetime.datetime(2020, 1, 1, 10, 0))

        for d in ("today", "2020-13-01"):
            with self.assertRaises(types.LesanaValueError):
                checker.load(d)

        v = checker.load(None)
        self.assertEqual(v, None)

        v = checker.auto(today)
        self.assertEqual(v, today)

    def test_date_auto(self):
        field_def = self._get_field_def('date')
        field_def['auto'] = 'creation'
        checker = types.LesanaDate(field_def, {})

        today = datetime.date.today()
        v = checker.empty()
        self.assertIsInstance(v, datetime.date)
        self.assertEqual(v, today)

        field_def['auto'] = False
        checker = types.LesanaDate(field_def, {})

        v = checker.empty()
        self.assertEqual(v, None)

        # auto=update fields should also be filled at creation time
        field_def['auto'] = 'update'
        checker = types.LesanaDate(field_def, {})

        v = checker.empty()
        self.assertIsInstance(v, datetime.date)
        self.assertEqual(v, today)

    def test_date_auto_update(self):
        field_def = self._get_field_def('date')
        field_def['auto'] = 'update'
        checker = types.LesanaDate(field_def, {})

        today = datetime.date.today()
        past = datetime.date(2016, 12, 10)
        # we pass a date in the past
        v = checker.auto(past)
        self.assertIsInstance(v, datetime.date)
        # and we want to get a date in the present
        self.assertEqual(v, today)

        # with auto=False we want our old date instead
        field_def['auto'] = False
        checker = types.LesanaDate(field_def, {})

        v = checker.auto(past)
        self.assertEqual(v, past)

        # and the same should happen with auto=creation
        field_def['auto'] = 'creation'
        checker = types.LesanaDate(field_def, {})

        v = checker.auto(past)
        self.assertEqual(v, past)

    def test_boolean(self):
        checker = types.LesanaBoolean(self._get_field_def('boolean'), {})

        v = checker.empty()
        self.assertEqual(v, None)

        v = checker.load(True)
        self.assertEqual(v, True)

        for d in ("maybe", "yes", "no"):
            with self.assertRaises(types.LesanaValueError):
                checker.load(d)

        v = checker.load(None)
        self.assertEqual(v, None)

        v = checker.auto(True)
        self.assertEqual(v, True)

    def test_file(self):
        checker = types.LesanaFile(self._get_field_def('file'), {})

        v = checker.empty()
        self.assertEqual(v, "")

        v = checker.load("relative/path/to/file")
        self.assertEqual(v, "relative/path/to/file")

        v = checker.load(None)
        self.assertEqual(v, None)

        # TODO: check for invalid file paths

        v = checker.auto("relative/path/to/file")
        self.assertEqual(v, "relative/path/to/file")

    def test_url(self):
        checker = types.LesanaURL(self._get_field_def('url'), {})

        v = checker.empty()
        self.assertEqual(v, "")

        v = checker.load("http://example.org")
        self.assertEqual(v, "http://example.org")

        v = checker.load(None)
        self.assertEqual(v, None)

        # TODO: check for invalid URLs

        v = checker.auto("http://example.org")
        self.assertEqual(v, "http://example.org")

    def test_yaml(self):
        checker = types.LesanaYAML(self._get_field_def('yaml'), {})

        v = checker.empty()
        self.assertEqual(v, None)

        some_data = {
            'anything': 'goes',
            'everything': 42
        }
        v = checker.load(some_data)
        self.assertEqual(v, some_data)

        v = checker.load(None)
        self.assertEqual(v, None)

        v = checker.auto(some_data)
        self.assertEqual(v, some_data)

    def test_list(self):
        field_def = self._get_field_def('yaml')
        # we use one type that is easy to check for correct validation
        field_def['list'] = 'int'
        checker = types.LesanaList(field_def, {'int': types.LesanaInt})

        v = checker.empty()
        self.assertEqual(v, [])

        some_data = [1, 2, 3]
        v = checker.load(some_data)
        self.assertEqual(v, some_data)

        v = checker.load(None)
        self.assertEqual(v, [])

        for d in (['hello'], 1):
            with self.assertRaises(types.LesanaValueError):
                checker.load(d)

        v = checker.auto(some_data)
        self.assertEqual(v, some_data)

    def test_list_unknown_subtype(self):
        field_def = self._get_field_def('yaml')
        # we use one type that is easy to check for correct validation
        field_def['list'] = 'int'
        checker = types.LesanaList(field_def, {'yaml': types.LesanaYAML})

        v = checker.empty()
        self.assertEqual(v, [])

        some_data = [1, 2, 3]
        v = checker.load(some_data)
        self.assertEqual(v, some_data)

        some_data = ["hello"]
        v = checker.load(some_data)
        self.assertEqual(v, some_data)

        v = checker.load(None)
        self.assertEqual(v, [])

        for d in (1, 1.0):
            with self.assertRaises(types.LesanaValueError):
                checker.load(d)


class testTypeIndexing(unittest.TestCase):
    def setUp(self):
        self.doc = xapian.Document()
        self.indexer = xapian.TermGenerator()

    def _get_field_def(self, type_name):
        return {
            'type': type_name,
            'name': 'test_field',
            'index': 'field',
            'sortable': True,
        }

    def test_base(self):
        checker = types.LesanaType(self._get_field_def('base'), {}, 16)

        checker.index(self.doc, self.indexer, "some string")

    def test_base_value_index_too_low(self):
        checker = types.LesanaType(self._get_field_def('base'), {}, 1)

        checker.index(self.doc, self.indexer, "some string")

        # TODO: check that the string has not been indexed

    def test_string(self):
        checker = types.LesanaString(self._get_field_def('string'), {}, 16)

        checker.index(self.doc, self.indexer, "some string")

    def test_text(self):
        checker = types.LesanaText(self._get_field_def('text'), {}, 16)

        checker.index(self.doc, self.indexer, "some string")

    def test_int(self):
        checker = types.LesanaInt(self._get_field_def('integer'), {}, 16)

        checker.index(self.doc, self.indexer, 1)

    def test_float(self):
        checker = types.LesanaFloat(self._get_field_def('float'), {}, 16)

        checker.index(self.doc, self.indexer, 1.5)

    def test_decimal(self):
        checker = types.LesanaDecimal(self._get_field_def('decimal'), {}, 16)

        checker.index(self.doc, self.indexer, decimal.Decimal('1.0'))

    def test_timestamp(self):
        checker = types.LesanaTimestamp(
            self._get_field_def('timestamp'), {}, 16
        )

        checker.index(self.doc, self.indexer, 1600000000)

    def test_datetime(self):
        checker = types.LesanaDatetime(self._get_field_def('datetime'), {}, 16)

        checker.index(self.doc, self.indexer, datetime.datetime.now())

    def test_date(self):
        checker = types.LesanaDate(self._get_field_def('date'), {}, 16)

        checker.index(self.doc, self.indexer, datetime.date.today())

    def test_boolean(self):
        checker = types.LesanaBoolean(self._get_field_def('boolean'), {}, 16)

        checker.index(self.doc, self.indexer, True)

    def test_url(self):
        checker = types.LesanaURL(self._get_field_def('url'), {}, 16)

        checker.index(self.doc, self.indexer, "http://example.org")

    def test_yaml(self):
        checker = types.LesanaYAML(self._get_field_def('yaml'), {}, 16)

        checker.index(self.doc, self.indexer, {'a': 1, 'b': 2})

    def test_list(self):
        field_def = self._get_field_def('yaml')
        # we use one type that is easy to check for correct validation
        field_def['list'] = 'int'
        checker = types.LesanaList(field_def, {'int': types.LesanaInt}, 16)

        checker.index(self.doc, self.indexer, ["some", "thing"])


if __name__ == '__main__':
    unittest.main()
